@@ -53,15 +53,18 @@
 
 #include <QtWidgets/private/qtwidgetsglobal_p.h>
 
-#ifndef QT_NO_LINEEDIT
 #include "private/qwidget_p.h"
 #include "QtWidgets/qlineedit.h"
+#if QT_CONFIG(toolbutton)
 #include "QtWidgets/qtoolbutton.h"
+#endif
 #include "QtGui/qtextlayout.h"
 #include "QtGui/qicon.h"
 #include "QtWidgets/qstyleoption.h"
 #include "QtCore/qbasictimer.h"
+#if QT_CONFIG(completer)
 #include "QtWidgets/qcompleter.h"
+#endif
 #include "QtCore/qpointer.h"
 #include "QtCore/qmimedata.h"
 
@@ -69,12 +72,14 @@
 
 #include <algorithm>
 
+QT_REQUIRE_CONFIG(lineedit);
+
 QT_BEGIN_NAMESPACE
 
 class QLineEditPrivate;
 
 // QLineEditIconButton: This is a simple helper class that represents clickable icons that fade in with text
-
+#if QT_CONFIG(toolbutton)
 class Q_AUTOTEST_EXPORT QLineEditIconButton : public QToolButton
 {
     Q_OBJECT
@@ -89,8 +94,8 @@ public:
 #endif
 
 protected:
-    void actionEvent(QActionEvent *e) Q_DECL_OVERRIDE;
-    void paintEvent(QPaintEvent *event) Q_DECL_OVERRIDE;
+    void actionEvent(QActionEvent *e) override;
+    void paintEvent(QPaintEvent *event) override;
 
 private slots:
     void updateCursor();
@@ -103,6 +108,7 @@ private:
 
     qreal m_opacity;
 };
+#endif // QT_CONFIG(toolbutton)
 
 class Q_AUTOTEST_EXPORT QLineEditPrivate : public QWidgetPrivate
 {
@@ -135,7 +141,7 @@ public:
         dragEnabled(0), clickCausedFocus(0), hscroll(0), vscroll(0),
         alignment(Qt::AlignLeading | Qt::AlignVCenter),
         leftTextMargin(0), topTextMargin(0), rightTextMargin(0), bottomTextMargin(0),
-        lastTextSize(0)
+        lastTextSize(0), mouseYThreshold(0)
     {
     }
 
@@ -149,6 +155,7 @@ public:
     QPointer<QAction> selectAllAction;
 #endif
     void init(const QString&);
+    void initMouseYThreshold();
 
     QRect adjustedControlRect(const QRect &) const;
 
@@ -200,11 +207,11 @@ public:
 #endif
     void _q_selectionChanged();
     void _q_updateNeeded(const QRect &);
-#ifndef QT_NO_COMPLETER
+#if QT_CONFIG(completer)
     void _q_completionHighlighted(const QString &);
 #endif
     QPoint mousePressPos;
-#ifndef QT_NO_DRAGANDDROP
+#if QT_CONFIG(draganddrop)
     QBasicTimer dndTimer;
     void drag();
 #endif
@@ -234,17 +241,23 @@ public:
     int effectiveRightTextMargin() const;
 
 private:
-    typedef QPair<QLineEdit::ActionPosition, int> PositionIndexPair;
+    struct SideWidgetLocation {
+        QLineEdit::ActionPosition position;
+        int index;
 
-    PositionIndexPair findSideWidget(const QAction *a) const;
+        bool isValid() const { return index >= 0; }
+    };
+    friend class QTypeInfo<SideWidgetLocation>;
+
+    SideWidgetLocation findSideWidget(const QAction *a) const;
 
     SideWidgetEntryList leadingSideWidgets;
     SideWidgetEntryList trailingSideWidgets;
     int lastTextSize;
+    int mouseYThreshold;
 };
 Q_DECLARE_TYPEINFO(QLineEditPrivate::SideWidgetEntry, Q_PRIMITIVE_TYPE);
-
-#endif // QT_NO_LINEEDIT
+Q_DECLARE_TYPEINFO(QLineEditPrivate::SideWidgetLocation, Q_PRIMITIVE_TYPE);
 
 QT_END_NAMESPACE
 
